@@ -3,6 +3,7 @@
 #include "config.h"
 #include "debug.h"
 #include <qapplication.h>
+#include <qeventloop.h>
 #include <qmessagebox.h>
 #include <qcstring.h>
 #include <qdatastream.h>
@@ -21,7 +22,7 @@
 static QStringList CANNA_AF_UNIX_FILES;
 
 CannaEngine::CannaEngine() 
-  : _contxt(0), _convertflag(FALSE)
+  : _contxt(0), _convertflag(FALSE), _connectimer(0)
 {
   CANNA_AF_UNIX_FILES << "/tmp/.iroha_unix/IROHA" << "/var/run/.iroha_unix/IROHA";
   _sock = new QSocket();
@@ -43,6 +44,7 @@ CannaEngine::name() const
   return QString("Canna");
 }
 
+
 bool
 CannaEngine::isTCPConnectionSupported() const
 {
@@ -52,25 +54,24 @@ CannaEngine::isTCPConnectionSupported() const
 
 // connect to Kanji server
 bool
-CannaEngine::init() 
+CannaEngine::init()
 {
   DEBUG_TRACEFUNC();
   _sock->close();
   
-  if ( Config::readBoolEntry("_chkremote", FALSE) ) {
+  if ( Config::readBoolEntry("_grpremote", FALSE) ) {
     // INET Domain
     qDebug("INET Domain socket");
     _sock->connectToHost(Config::readEntry("_edtsvrname", "localhost"), 
 			 Config::readEntry("_edtport", "0").toUInt());
+    _connectimer = startTimer(3000);
 
   } else {
     // UNIX Domain
     qDebug("UNIX Domain socket");
 
-    int    fd;
-    struct sockaddr_un  addr;
-    
-    if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
+    int  fd = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (fd < 0) {
       QMessageBox::critical(0, "Kanji Server Error", 
 			    "Socket Create Error",
 			    QMessageBox::Ok | QMessageBox::Default, 0);
@@ -79,6 +80,7 @@ CannaEngine::init()
 
     int i;
     for (i = 0; i < (int)CANNA_AF_UNIX_FILES.count(); ++i) {
+      struct sockaddr_un  addr;
       memset(&addr, 0, sizeof(addr));
       addr.sun_family = AF_UNIX;
       strncpy(addr.sun_path, CANNA_AF_UNIX_FILES[i].latin1(), sizeof(addr.sun_path) - 1);
@@ -107,7 +109,14 @@ CannaEngine::init()
 void 
 CannaEngine::cleanup()
 {
+  DEBUG_TRACEFUNC();
   _sock->close();
+  _contxt = 0;
+  _convertflag = FALSE;
+  if (_connectimer > 0) {
+    killTimer(_connectimer);
+    _connectimer = 0;
+ }
 }
 
 
@@ -124,19 +133,13 @@ void
 CannaEngine::slotDetectError() 
 {
   DEBUG_TRACEFUNC();
-  QMessageBox::critical(0, "Kanji Server Error", 
-			"Socket Error!\nConnects to localhost by UNIX domain.",
-			QMessageBox::Ok | QMessageBox::Default, 0);
+  cleanup();
+  Config::writeEntry("_grpremote", FALSE);
   
-  Config::writeEntry("_chkremote", FALSE);
-
-  static int errcount = 0;
-  if (++errcount > 2) {
-    qApp->quit();
-    return;
-  }
-
-  init();
+  QMessageBox::critical(0, "Kanji Server Error", 
+			tr("Canna Ȥ̿ǥ顼ȯޤ\n"
+			   "ʴѴߤޤ"),
+			QMessageBox::Ok | QMessageBox::Default, 0);
 }
 
 
@@ -144,19 +147,13 @@ void
 CannaEngine::slotConnectionClosed()
 {
   DEBUG_TRACEFUNC();
-  QMessageBox::critical(0, "Kanji Server Connection Closed",
-			"Socket Connection Closed!\nConnects to localhost by UNIX domain.",
-			QMessageBox::Ok | QMessageBox::Default, 0);
-  
-  Config::writeEntry("_chkremote", FALSE);
+  cleanup();
+  Config::writeEntry("_grpremote", FALSE);
 
-  static int errcount = 0;
-  if (++errcount > 2) {
-    qApp->quit();
-    return;
-  }
-
-  init();
+  QMessageBox::warning(0, "Kanji Server Connection Closed",
+		       tr("Canna ȤΥåǤޤ\n"
+			  "ʴѴߤޤ"),
+		       QMessageBox::Ok | QMessageBox::Default, 0);
 }
 
 
@@ -179,7 +176,7 @@ CannaEngine::sendInitialize()
   QTime t;
   t.start();
   while (_sock->bytesAvailable() != 4) {
-    qApp->processEvents();
+    qApp->eventLoop()->processEvents(QEventLoop::ExcludeUserInput);
     if (t.elapsed() > 2000) {
       qWarning("No response!");
       return;
@@ -212,12 +209,11 @@ CannaEngine::recvInitializeReply()
   int res;
   ds >> res;
   if (res == -1 || res == -2) {
-    int ret = QMessageBox::critical(0, "Protocol version error", 
-				    "Protocol version error",
-				    QMessageBox::Ok | QMessageBox::Default, 0);
-    if (ret) {
-      qApp->quit();
-    }
+    cleanup();
+    QMessageBox::critical(0, "Protocol version error", 
+			  "Canna protocol version error",
+			  QMessageBox::Ok | QMessageBox::Default, 0);
+    return;
   } 
 
   QDataStream dst(respack, IO_ReadOnly);
@@ -286,12 +282,10 @@ CannaEngine::recvNoticeGroupName()
   int8_t  stat;
   ds >> stat;
   if (stat) {
-    int ret = QMessageBox::critical(0, "mount dictionary error",
-				    "mount dictionary error",
-				    QMessageBox::Ok | QMessageBox::Default, 0);
-    if (ret)
-      qApp->quit();
-  
+    cleanup();
+    QMessageBox::critical(0, "mount dictionary error",
+			  "Canna mount dictionary error",
+			  QMessageBox::Ok | QMessageBox::Default, 0);  
   } else {
     qDebug("notice group name successful.");
   }
@@ -415,10 +409,10 @@ CannaEngine::recvMountDictionaryReply()
   int8_t  stat;
   ds >> stat;
   if (stat) {
-    int ret = QMessageBox::critical(0, "mount dictionary error",
-				    "mount dictionary error",
-				    QMessageBox::Ok | QMessageBox::Default, 0);
-    if (ret) qApp->quit();
+    cleanup();
+    QMessageBox::critical(0, "mount dictionary error",
+			  "Canna mount dictionary error",
+			  QMessageBox::Ok | QMessageBox::Default, 0);
     return FALSE;
   }  
   qDebug("mount dictionary successful.");
@@ -430,6 +424,9 @@ bool
 CannaEngine::beginConvert(const QString& hira, QStringList& kanji, QStringList& yomigana)
 {
   DEBUG_TRACEFUNC();
+  if ( _sock->state() != QSocket::Connected )
+    return FALSE;
+
   if ( _convertflag ) {
     qWarning("%s:_convertflag incorrect", __func__);
     return FALSE;
@@ -440,7 +437,8 @@ CannaEngine::beginConvert(const QString& hira, QStringList& kanji, QStringList&
   ds.setByteOrder(QDataStream::BigEndian);
 
   qDebug("Yomigana size(QCString): %d", hira.local8Bit().length());
-  QByteArray str = eucToUint16(hira.local8Bit());
+  //QByteArray str = eucToUint16(hira.local8Bit());
+  QByteArray str = eucToUint16(QTextCodec::codecForTr()->fromUnicode(hira));
   qDebug("Yomigana size(uint16 array): %d", str.size());
 
   ds << (u_int8_t)RID_BEGIN_CONVERT << (u_int8_t)0 << (u_int16_t)(str.size()+6) << 0L << _contxt;
@@ -526,6 +524,9 @@ void
 CannaEngine::endConvert(const QStringList &)
 {
   DEBUG_TRACEFUNC();
+  if ( _sock->state() != QSocket::Connected )
+    return;
+
   if ( !_convertflag ) {
     qDebug("%s: unnecessary to end convert", __func__);
     return;
@@ -588,6 +589,9 @@ bool
 CannaEngine::getCandidate(int idx, QStringList& candidate)
 {
   DEBUG_TRACEFUNC("idx: %d", idx);
+  if ( _sock->state() != QSocket::Connected )
+    return FALSE;
+
   if ( !_convertflag )
     return FALSE;
  
@@ -595,22 +599,22 @@ CannaEngine::getCandidate(int idx, QStringList& candidate)
   QDataStream ds(pack, IO_WriteOnly);
   ds.setByteOrder(QDataStream::BigEndian);
 
-  ds << (u_int8_t)RID_GET_CANDIDACY_LIST << (u_int8_t)0 << (u_int16_t)6 << _contxt 
+  ds << (u_int8_t)RID_GET_CANDIDATE_LIST << (u_int8_t)0 << (u_int16_t)6 << _contxt 
      << (int16_t)idx << (u_int16_t)4096;
   _sock->writeBlock(pack.data(), pack.size());
 
-  if ( !waitForReply(RID_GET_CANDIDACY_LIST) ) {
+  if ( !waitForReply(RID_GET_CANDIDATE_LIST) ) {
     slotDetectError();
     return FALSE;
   }
 
-  candidate = recvGetCandidacyListReply();
+  candidate = recvGetCandidateListReply();
   return TRUE;
 }
 
 
 QStringList
-CannaEngine::recvGetCandidacyListReply()
+CannaEngine::recvGetCandidateListReply()
 {
   DEBUG_TRACEFUNC();
   uint datalen = _sock->size();
@@ -634,7 +638,7 @@ CannaEngine::recvGetCandidacyListReply()
   int16_t ncand;
   ds >> ncand;
   if (ncand < 0) {
-    qFatal("GetCandidacyList error.  %s:%d", __FILE__, __LINE__);
+    qFatal("GetCandidateList error.  %s:%d", __FILE__, __LINE__);
     return QStringList();
   }
   qDebug("Number of Candidates: %d", ncand);
@@ -735,12 +739,19 @@ bool
 CannaEngine::resizeSegment(int idx, int len, QStringList& kanji, QStringList& yomigana)
 {
   DEBUG_TRACEFUNC("idex: %d  len: %d", idx, len);
+  if ( _sock->state() != QSocket::Connected )
+    return FALSE;
 
   if ( !_convertflag ) {
     Q_ASSERT(0);
     return FALSE;
   }
 
+  if (len < 0) {
+    int length = getYomi(0).length();
+    len = (len == LENGTHEN_1CHAR) ? length + 1 : length - 1;
+  }
+
   QByteArray  pack;
   QDataStream ds(pack, IO_WriteOnly);
   ds.setByteOrder(QDataStream::BigEndian);
@@ -952,4 +963,26 @@ CannaEngine::uint16ToEuc(const QByteArray& src)
 }
 
 
+void
+CannaEngine::timerEvent(QTimerEvent* e)
+{
+  DEBUG_TRACEFUNC();
+  if (_connectimer > 0 && e->timerId() == _connectimer) {
+    killTimer(_connectimer);
+    _connectimer = 0;
+    
+    if (_sock->state() != QSocket::Connected) {
+      // Detects error
+      cleanup();
+      Config::writeEntry("_grpremote", FALSE);
+      
+      QMessageBox::warning(0, "Kanji Server Connection Error",
+			   tr("Canna ȤΥåΩǤޤ\n"
+			      "ʴѴߤޤ"),
+			   QMessageBox::Ok | QMessageBox::Default, 0);
+    }
+  }
+}
+
+
 EXPORT_KANJIENGINE(CannaEngine)
