@@ -10,6 +10,8 @@
 #include <qregexp.h>
 using namespace Kimera;
 
+const QKeyEvent DictHiragana::SEPARATOR_CHAR = QKeyEvent((QEvent::Type)QEvent::KeyPress, 0, 0, 0);
+
 DictHiragana::DictHiragana() : _dicthira(QDict<QString>(17, FALSE)),
 			       _dictkata(QDict<QString>()),
 			       _dicthankana(QDict<QString>()),
@@ -36,6 +38,7 @@ DictHiragana::~DictHiragana() { }
 void
 DictHiragana::init()
 {
+  DEBUG_TRACEFUNC();
   _dicthira.clear();
   _dictkata.clear();
   _dicthankana.clear();
@@ -53,7 +56,7 @@ DictHiragana::init()
   copyDictFile("kanainput.dic");
   copyDictFile("dakuten.dic");
 
-  QString dictdir = Config::dictionaryPath();
+  QString dictdir = Config::dictionaryDirPath();
   initDict(_dicthira, dictdir + "/hiragana.dic");          // Ҥ餬ʼ
   initDict(_dictkata, dictdir + "/katakana.dic");          // ʼ
   initDict(_dicthankana, dictdir + "/hankakukana.dic");    // Ⱦѥʼ
@@ -90,6 +93,7 @@ DictHiragana::init()
 void
 DictHiragana::initDict(QDict<QString>& dict, const QString& dictfile, bool reverse_dict)
 {
+  DEBUG_TRACEFUNC();
   QFile file(dictfile);
   if ( !file.open(IO_ReadOnly) ) {
     QMessageBox::critical(0, "File open error", "Cannot open file: " + dictfile,
@@ -102,25 +106,24 @@ DictHiragana::initDict(QDict<QString>& dict, const QString& dictfile, bool rever
   stream.setEncoding(QTextStream::Latin1);
 
   while ( !stream.eof() ) {
-    QString line;
-    line = QString::fromLocal8Bit(stream.readLine());
-    
-    if ( !line.isEmpty() && !line.contains(QRegExp("^[ \t]")) && 
-	 !line.contains(QRegExp("^##")) ) {
+    //QString line = QString::fromLocal8Bit(stream.readLine());
+    QString line = QObject::tr(stream.readLine());
+    if ( !line.isEmpty() && !line.contains(QRegExp("^##")) ) {
       QString keystr, itemstr;
       int idx = line.find(QRegExp("[ \t]"));
       if (idx > 0) {
 	keystr = line.left(idx);	
 	line.remove(0, keystr.length());
 	itemstr = line.replace(QRegExp("[ \t]"), "");
-	if ( !reverse_dict ) {
-	  if ( !dict.find(keystr) )
-	    dict.replace(keystr, new QString(itemstr));  
-	
-	} else {
-	  // reverse-dictionary
-	  if ( !dict.find(itemstr) )
-	    dict.insert(itemstr, new QString(keystr));
+	if (!keystr.isEmpty() && !itemstr.isEmpty()) {
+	  if ( !reverse_dict ) {
+	    if ( !dict.find(keystr) )
+	      dict.insert(keystr, new QString(itemstr));
+	  } else {
+	    // reverse-dictionary
+	    if ( !dict.find(itemstr) )
+	      dict.insert(itemstr, new QString(keystr));
+	  }
 	}
       }
     }
@@ -136,34 +139,15 @@ DictHiragana::initDict(QDict<QString>& dict, const QString& dictfile, bool rever
 QString
 DictHiragana::convertToHira(const QString& src) const
 {
+  DEBUG_TRACEFUNC("src: %s", src.local8Bit().data());
   // check parameter
   if ( src.isEmpty() )
     return QString::null;
 
-  QString dest;
-  if (src.local8Bit().length() != src.length()) {
-    // Ѵ
-    if (src.length() == 1) 
-      return src;
-
-    uint i = 0;
-    while (i < src.length()) {
-      QString* pstr;
-      pstr = _dictdakuten.find( src.mid(i, 2) );
-      
-      if ( pstr ) {
-	dest += *pstr;
-	i++;
-      } else {
-	dest += src.constref(i);
-      }
-      i++;
-    }
-    
-    return dest;
-
-  }  
+  if (src.local8Bit().length() != src.length())
+    return replaceDakutenChar(src);
   
+  QString dest;
   uint index = 0;
   while (index < src.length()) {
     for (int len = 4; len > 0; --len) {
@@ -213,25 +197,7 @@ DictHiragana::convertToHira(const QString& src) const
       }
     }
   }
-  
-  return dest;
-}
 
-
-// convertToHira ƤӽФ塤Ǹʸ'n'ä '' Ѵ
-//
-QString
-DictHiragana::forceConvtToHira(const QString& src) const
-{
-  if ( src.isEmpty() )
-    return QString::null;
-
-  QString dest = convertToHira(src);
-  if (dest.right(1) == "n" || dest.right(1) == "N") {
-    dest.remove(dest.length() - 1, 1);
-    dest += QString::fromLocal8Bit("");
-  }
-  
   return dest;
 }
 
@@ -240,11 +206,13 @@ DictHiragana::forceConvtToHira(const QString& src) const
 QString
 DictHiragana::convertToKata(const QString& src) const
 {
+  DEBUG_TRACEFUNC("src: %s", src.local8Bit().data());
   Q_ASSERT(!_dictkata.isEmpty());
 
   QString dest;
   // check parameter
   if (src.local8Bit().length() == src.length()) {
+    // Case of Romaji
     QString hira = convertToHira(src);
     for (int j = 0; j < (int)hira.length(); j++) {
       QString* pstr = _dictkata.find( hira.constref(j) );
@@ -254,15 +222,15 @@ DictHiragana::convertToKata(const QString& src) const
 	dest += hira.constref(j);
       }
     }
-    
   } else {
     // Hiragana -> Katakana
-    for (int i = 0; i < (int)src.length(); i++) {
-      QString* pstr = _dictkata.find( src.constref(i) );
+    QString str = replaceDakutenChar(src);
+    for (int i = 0; i < (int)str.length(); i++) {
+      QString* pstr = _dictkata.find( str.constref(i) );
       if ( pstr ) {
 	dest += *pstr;
       } else {
-	dest += src.constref(i);
+	dest += str.constref(i);
       }
     }
   }
@@ -271,18 +239,11 @@ DictHiragana::convertToKata(const QString& src) const
 }
 
 
-// parameter: Roma string (only ASCII characters) or Hiragana string
-QString
-DictHiragana::forceConvtToKata(const QString& src) const
-{
-  return convertToKata( forceConvtToHira(src) );
-}
-
-
 // parameter: Roma string (only ASCII characters) or Hiragana string
 QString
 DictHiragana::convertToHankakuKana(const QString& src) const
 {
+  DEBUG_TRACEFUNC();
   Q_ASSERT(!_dicthankana.isEmpty());
 
   QString dest;
@@ -314,18 +275,11 @@ DictHiragana::convertToHankakuKana(const QString& src) const
 }
 
 
-// parameter: Roma string (only ASCII characters) or Hiragana string
-QString
-DictHiragana::forceConvtToHankakuKana(const QString& src) const
-{
-  return convertToHankakuKana( forceConvtToHira(src) );
-}
-
-
 // parameter: Roma string (only ASCII characters)
 QString
 DictHiragana::convertToZenkakuEisu(const QString& src) const
-{  
+{
+  DEBUG_TRACEFUNC();
   Q_ASSERT(!_dictalphbt.isEmpty());
   Q_ASSERT(!_dictsymbol.isEmpty());
 
@@ -348,33 +302,41 @@ DictHiragana::convertToZenkakuEisu(const QString& src) const
 
 
 QString
-DictHiragana::convert(const QString& roma, const InputMode& mode) const
+DictHiragana::convertString(const QString& hira, const InputMode& mode) const
 {
+  DEBUG_TRACEFUNC("hira: %s  mode:0x%x", hira.local8Bit().data(), mode.id());
+  if (hira.isEmpty())
+    return QString::null;
+
   QString res;
   switch (mode.id() & Mode_ModeMask) {
   case Mode_Hiragana:
-    res = convertToHira(roma);
+    res = convertToHira(hira);
     break;
     
   case Mode_Katakana:
-    res = convertToKata(roma);
+    res = convertToKata(hira);
     break;
-
+    
   case Mode_HankakuKana:
-    res = convertToHankakuKana(roma);
+    res = convertToHankakuKana(hira);
     break;
 
   case Mode_ZenkakuEisu:
-    if (roma.local8Bit().length() != roma.length()) {
+    if (hira.local8Bit().length() != hira.length()) {
       // Multibyte chars
-      res = convertToZenkakuEisu( reverseConvt(roma) );
+      res = convertToZenkakuEisu( reverseConvt(hira) );
     } else {
-      res = convertToZenkakuEisu(roma);
+      res = convertToZenkakuEisu(hira);
     }
     break;
     
   case Mode_HankakuEisu:
-    res = reverseConvt(roma);
+    if (hira.local8Bit().length() != hira.length()) {
+      res = reverseConvt(hira);
+    } else {
+      res = hira;
+    }
     break;
 
   default:
@@ -386,41 +348,76 @@ DictHiragana::convert(const QString& roma, const InputMode& mode) const
 }
 
 
+//
+// Ǹʸ ('n' + SEPARATOR_CHAR) ä, '' Ѵ
+//
 QString
-DictHiragana::forceConvt(const QString& roma, const InputMode& mode) const
+DictHiragana::convert(const QPtrList<QKeyEvent>& kevlst, const InputMode& mode, bool tenkey_hankaku) const
 {
+  DEBUG_TRACEFUNC("kevlst.count: %u  mode: 0x%x  tenkey_hankaku: %d", kevlst.count(), mode.id(), tenkey_hankaku);
+  if ( !kevlst.count() )
+    return QString::null;
+  
   QString res;
-  switch (mode.id() & Mode_ModeMask) {
-  case Mode_Hiragana:
-    res = forceConvtToHira(roma);
-    break;
-    
-  case Mode_Katakana:
-    res = forceConvtToKata(roma);
-    break;
+  QPtrListIterator<QKeyEvent> it(kevlst);
+  QKeyEvent* kev;  
 
-  case Mode_HankakuKana:
-    res = forceConvtToHankakuKana(roma);
-    break;
-    
-  case Mode_ZenkakuEisu:
-    if (roma.local8Bit().length() != roma.length()) {
-      // Multibyte chars
-      res = convertToZenkakuEisu( reverseConvt(roma) );     
-    } else {
-      res = convertToZenkakuEisu(roma);
+  if ((mode.id() & Mode_KanaInput) && !(mode.id() & (Mode_ZenkakuEisu | Mode_HankakuEisu))) {
+    QString str;
+    while ( (kev = it.current()) ) {
+      ++it;
+      if (tenkey_hankaku && (kev->state() & Qt::Keypad)) {
+	res += convertString(str, mode);   // Ѵ
+	str = QString::null;
+	res += kev->text();
+      } else {
+	str += convertKey(*kev, mode);
+      }
     }
-    break;
+    res += convertString(str, mode);
+
+  } else if (mode.id() & (Mode_ZenkakuEisu | Mode_HankakuEisu)) {
+    QString cstr;
+    while ( (kev = it.current()) ) {
+      ++it;
+      cstr += kev->text();
+    }
+    res = convertString(cstr, mode);
     
-  case Mode_HankakuEisu:
-    res = reverseConvt(roma);
-    break;
+  } else {
+    QString cstr;
+    while ( (kev = it.current()) ) {
+      ++it;
+      if (tenkey_hankaku && (kev->state() & Qt::Keypad)) {
+	res += convertString(cstr, mode);
+	cstr = QString::null;
+	res += kev->text();
+      } else if (kev->state() & Qt::MetaButton) {
+	// Couples dakuten-char
+	QString s = convertString(cstr, mode);
+	if (s.length() > 1) {
+	  res += s.left(s.length() - 1);
+	}
+	res += convertString(s.right(1) + convertKey(*kev, mode), mode);
+	cstr = QString::null;
+      } else {
+	if ( !kev->text().isEmpty() ) {   // means it's not SEPARATOR_CHAR
+	  cstr += kev->text();
+	} else {
+	  res += convertString(cstr, mode); // Check next char
+	  cstr = QString::null;
+	}
+      }
+    }
+    res += convertString(cstr, mode);
 
-  default:
-    Q_ASSERT(0);
-    break;
+    // Check last KeyEvent
+    if (kevlst.getLast()->text().isEmpty() && (res.right(1) == "n" || res.right(1) == "N")) {
+      res.truncate(res.length() - 1);
+      res += convertString("nn", mode);
+    }
   }
-
+  
   return res;
 }
 
@@ -430,30 +427,32 @@ DictHiragana::forceConvt(const QString& roma, const InputMode& mode) const
 //  params: Input mode
 //  return: Hiragana string
 QString
-DictHiragana::convert(const QKeyEvent& key, const InputMode& mode) const
+DictHiragana::convertKey(const QKeyEvent& key, const InputMode& mode) const
 {
-  if ( !(mode.id() & Mode_KanaInput) ) {
-    return QString::null;
-
-  } else if (mode.id() & Mode_ZenkakuEisu) {
+  DEBUG_TRACEFUNC();
+  if (mode.id() & Mode_ZenkakuEisu) {
     return convertToZenkakuEisu(key.text());
   }
-
-  Q_ASSERT(!_dictkanainp.isEmpty());
-  QString keystr;
-  if (key.state() == Qt::NoButton) {
-    keystr = key.text();
-    
-  } else if (key.state() == Qt::ShiftButton) { 
-    keystr = QString("Shift+") + key.text();
-    
-  } else if (key.state() == Qt::Keypad) {
-    keystr = QString("Keypad+") + key.text();
-    
-  } else {
+  if (mode.id() & Mode_HankakuEisu) {
+    return key.text();
+  }
+  if ((mode.id() & Mode_RomaInput) && !(key.state() & Qt::MetaButton)) {
     return QString::null;
   }
   
+  QString keystr;
+  if (key.state() == Qt::NoButton) {
+    keystr = key.text();
+  } else if (key.state() & Qt::ShiftButton) { 
+    keystr = QString("Shift+") + key.text();
+  } else if (key.state() & Qt::Keypad) {
+    keystr = QString("Keypad+") + key.text();
+  } else if (key.state() & Qt::MetaButton) {
+    keystr = QString("Meta+") + key.text();
+  } else {
+    return QString::null;
+  }
+
   QString* pres = _dictkanainp[keystr];
   if ( !pres )
     return QString::null;
@@ -467,6 +466,10 @@ DictHiragana::convert(const QKeyEvent& key, const InputMode& mode) const
   case Mode_Katakana:
     res = convertToKata(*pres);
     break;
+
+  case Mode_HankakuKana:
+    res = convertToHankakuKana(*pres);
+    break;
     
   default:
     Q_ASSERT(0);
@@ -480,41 +483,71 @@ DictHiragana::convert(const QKeyEvent& key, const InputMode& mode) const
 QString
 DictHiragana::reverseConvt(const QString& hira) const
 {
+  DEBUG_TRACEFUNC();
   QString res;
-  int i = 0;
-  while (i < (int)hira.length()) {
+  for (int i = 0; i < (int)hira.length(); ++i) {
     QChar c = hira[i];
     if ( c.latin1() ) {   // Check ascii or multibyte-char
       // Ascii char
       res += c;
-      ++i;
     } else {
       // Multibyte char
-      QString* pstr;
-      int len;
-      for (len = 2; len > 0; --len)
-	if ( (pstr = _reversedict.find(hira.mid(i, len))) )
-	  break;
-      
-      if (pstr) {
-	res += *pstr;
-	i += len;
-      } else {
-	res += c;
-	++i;
+      QString* pstr = _reversedict.find(hira.mid(i, 1));
+      res += pstr ? *pstr : c;
+    }
+  } 
+  return (res == hira) ? res : reverseConvt(res);
+}
+
+
+//
+// Create new QKeyEventList
+//   Auto-deletion of QPtrList class (kevlst) must be enabled
+void
+DictHiragana::reverse(const QString& yomi, const InputMode& mode, bool tenkey_hankaku, QPtrList<QKeyEvent>& kevlst) const
+{
+  DEBUG_TRACEFUNC();
+
+  kevlst.clear();
+  if (yomi.isEmpty())
+    return;
+
+  if ((mode.id() & Mode_KanaInput) && (mode.id() & (Mode_Hiragana | Mode_Katakana | Mode_HankakuKana))) {
+    Q_ASSERT(0);
+    return;
+  }
+
+  QRegExp rx("[0-9+*/.-]");
+  for (int i = 0; i < (int)yomi.length(); ++i) {
+    if (tenkey_hankaku && rx.exactMatch(yomi.mid(i, 1))) {
+      // Tenkey 
+      kevlst.append(new QKeyEvent(QEvent::KeyPress, 0, 0, Qt::Keypad, yomi.mid(i, 1)));
+    } else if ((mode.id() & Mode_HankakuKana) && (yomi.mid(i, 1) == QObject::tr("") || yomi.mid(i, 1) == QObject::tr(""))) {
+      // Dakuten or Handakuten
+      if (kevlst.count() > 1 && kevlst.last()->text().isEmpty()) {  // SEPARATOR_CHAR
+	kevlst.remove(kevlst.count() - 1);
+      }
+      QString cstr = convertString(yomi.mid(i, 1), Mode_HankakuEisu);
+      kevlst.append(new QKeyEvent(QEvent::KeyPress, 0, 0, Qt::MetaButton, cstr));  // Exceptional key event
+    } else {
+      QString cstr = convertString(yomi.mid(i, 1), Mode_HankakuEisu);
+      for (int j = 0; j < (int)cstr.length(); ++j) {
+	kevlst.append(new QKeyEvent(QEvent::KeyPress, 0, 0, 0, cstr.mid(j, 1)));
       }
     }
+    
+    if (i != (int)yomi.length() - 1)
+      kevlst.append(new QKeyEvent(DictHiragana::SEPARATOR_CHAR));  // Insert SEPARATOR_CHAR
   }
-  
-  return (res == hira) ? res : reverseConvt(res);
 }
 
 
 void
 DictHiragana::copyDictFile(const QString dicname)
 {
+  DEBUG_TRACEFUNC();
   QFile src(QString(INSTALL_PATH "/dic/") + dicname);
-  QDir  dir(Config::dictionaryPath());
+  QDir  dir(Config::dictionaryDirPath());
   QFile dst( dir.filePath(dicname) );
 
   if( !dst.exists() ) {  
@@ -535,3 +568,28 @@ DictHiragana::copyDictFile(const QString dicname)
     tsdst << tssrc.read();
   }
 }
+
+
+QString
+DictHiragana::replaceDakutenChar(const QString& str) const
+{
+  if (str.length() <= 1) 
+    return str;
+  
+  QString dest;
+  uint i = 0;
+  while (i < str.length()) {
+    QString* pstr;
+    pstr = _dictdakuten.find( str.mid(i, 2) );
+    
+    if ( pstr ) {
+      dest += *pstr;
+      ++i;
+    } else {
+      dest += str.constref(i);
+    }
+    ++i;
+  }
+  
+  return dest;
+}
