@@ -9,6 +9,7 @@
 #include <qevent.h>
 #include <qmessagebox.h>
 #include <qkeysequence.h>
+#include <qtimer.h>
 #include <ctype.h>
 #include <string.h>
 #include <X11/Xlib.h>
@@ -24,13 +25,14 @@ enum {
 #undef KeyPress
 #undef KeyRelease
 
-void xEvent2XEvent(Display* dis, const void* src, XEvent& e);
+void xEvent2XEvent(Display* dis, const void* src, unsigned int serial, XEvent& e);
+
+const char *const SERVER_ATOM_STRING = "@server=kimera";
 
 
 InputMethod::InputMethod() : _cltwinstack(QValueStack<Window>()),
 			     _crnt_im(0),
 			     _crnt_ic(0),
-			     _xim_servers(0),
 			     _server_atom(0),
 			     _xim_xconnect(0),
 			     _xim_protocol(0),
@@ -39,21 +41,18 @@ InputMethod::InputMethod() : _cltwinstack(QValueStack<Window>()),
 			     _transport(0),
 			     _compound_text(0),
 			     _kimera_atom(0),
-			     _preedit_str()
+			     _inputon(FALSE)
 {
-  _xim = new XIMethod();
+  QTimer::singleShot(0, this, SLOT(init()));
 
+  _xim = new XIMethod();
   _buffer = new QBuffer();
   _buffer->open( IO_WriteOnly | IO_Append );
 
   // Create instance KanjiConvert class
   _kanjiconvt = new KanjiConvert();
 
-  connect(_kanjiconvt, SIGNAL(decideSegments(const QString&)),
-	  this,  SLOT(slotDecideSegments(const QString&)));
-
-  connect(this, SIGNAL(setInputMode(const InputMode&)), 
-	  _kanjiconvt,  SLOT(setInputMode(const InputMode&)));
+  connect(_kanjiconvt, SIGNAL(decideSegments(const QString&)), this,  SLOT(slotDecideSegments(const QString&)));
 }
 
 
@@ -70,15 +69,11 @@ void
 InputMethod::init()
 {
   DEBUG_TRACEFUNC();
-  _kanjiconvt->init();
-
   Q_CHECK_PTR(qt_xdisplay());
   qDebug("IMS Window ID: %lu", winId());
 
   // Create Atom
-  _xim_servers = XInternAtom(qt_xdisplay(), "XIM_SERVERS", False);
-  qDebug("XIM_SERVERS: %lu", _xim_servers);
-  _server_atom = XInternAtom(qt_xdisplay(), "@server=kimera", False);
+  _server_atom = XInternAtom(qt_xdisplay(), SERVER_ATOM_STRING, False);
   qDebug("@server=kimera: %lu", _server_atom);
   _xim_xconnect = XInternAtom(qt_xdisplay(), "_XIM_XCONNECT", False);
   qDebug("_XIM_XCONNECT: %lu", _xim_xconnect);
@@ -95,60 +90,6 @@ InputMethod::init()
   _kimera_atom = XInternAtom(qt_xdisplay(), "KIMERA_ATOM", False);
   qDebug("KIMERA_ATOM: %lu", _kimera_atom);
  
-  Atom type;
-  int format;
-  ulong nitems;
-  ulong bytes_after;
-  u_int8_t* value = 0;
-
-  // Grab server
-  XGrabServer(qt_xdisplay());
-  
-  // Get the Atom contained in XIM_SERVERS Property
-  int res = XGetWindowProperty(qt_xdisplay(), qt_xrootwin(), _xim_servers,
-			       0, 1024, False, XA_ATOM, &type, 
-			       &format, &nitems, &bytes_after, &value); 
-  Q_ASSERT(res == Success);
-  Q_ASSERT(bytes_after == 0);
-  
-  // Compare the gotten Atom and XIM_SERVER Atom 
-  Atom tmpatom = _server_atom;
-  if (res != Success || type != XA_ATOM || format != 32 || !value ) {
-    // Register XIM_SERVER Atom
-    XChangeProperty(qt_xdisplay(), qt_xrootwin(), _xim_servers, 
-		    XA_ATOM, 32, PropModeReplace, 
-		    (u_int8_t *)&tmpatom, 1);
-    qDebug("No XIM_SERVER. Regiter IM server");
-    
-  } else {
-    ulong* atoms = (ulong*)value;
-    int i;
-    for (i = 0; i < (int)nitems; i++) {
-      if (atoms[i] == _server_atom) {  
-	// Already registered
-	qDebug("IM server registered already. No change.");
-	break;
-      }
-    }
-    
-    if (i == (int)nitems) {
-      // Register XIM_SERVER Atom
-      XChangeProperty(qt_xdisplay(), qt_xrootwin(), _xim_servers, 
-		      XA_ATOM, 32, PropModeAppend,  
-		      (u_int8_t *)&tmpatom, 1);
-      qDebug("No Kimera. Register IM server");
-    }
-  }
-
-  if ( value )
-    XFree((char*)value);
-
-  // Ungrab server
-  XUngrabServer(qt_xdisplay());
-
-  // Flush
-  XFlush(qt_xdisplay());
-
   // Set Selection XIM_SERVER Atom (@im=kimera)  
   XSetSelectionOwner(qt_xdisplay(), _server_atom, winId(), 0);
   Q_ASSERT(XGetSelectionOwner(qt_xdisplay(), _server_atom) == winId());
@@ -158,49 +99,31 @@ InputMethod::init()
 XIMStyle
 InputMethod::currentXIMStyle() const
 {
-  return  _xim->getInputStyle(_crnt_im);
+  return  _xim->getInputStyle(_crnt_im, _crnt_ic);
 }
 
 
 void
-InputMethod::clearPreedit()
-{
-  DEBUG_TRACEFUNC();
-  if (_preedit_str.isEmpty() || _crnt_im <= 0 || _crnt_ic <= 0) {
-    // Do nothing
-    return;
-
-  } else if (_xim->getInputStyle(_crnt_im) != ON_THE_SPOT_STYLE) {
-    qDebug("Not on-the-spot style");
-    return;
-  }
-  
-  sendXIMPreeditDraw(_crnt_im, _crnt_ic);
-  _preedit_str = QString::null;
-}
-
-
-void
-InputMethod::appendPreeditString(const QString& str, FeedbackType t)
+InputMethod::sendPreeditString(int caret, const QStringList& strlst, int attention_idx)
 { 
   DEBUG_TRACEFUNC();
-  if (str.isEmpty() || _crnt_im <= 0 || _crnt_ic <= 0) {
+  if (!_crnt_im || !_crnt_ic) {
     // Do nothing
     return;
-
-  } else if (_xim->getInputStyle(_crnt_im) != ON_THE_SPOT_STYLE) {
-    qDebug("Not on-the-spot style");
-    return;
   }
   
   QByteArray  feedback;
   QDataStream ds(feedback, IO_WriteOnly);
   ds.setByteOrder(sysByteOrder());
-  for (int i = 0; i < (int)str.length(); ++i)
-    ds << (int)t;
+  for (int i = 0; i < (int)strlst.count(); ++i) {
+    int feed = (i == attention_idx) ? FT_REVERSE : FT_UNDERLINE;
+    for (int j = 0; j < (int)strlst[i].length(); ++j)
+      ds << feed;
+  }
 
-  sendXIMPreeditDraw(_crnt_im, _crnt_ic, str, feedback);
-  _preedit_str += str;
+  static int prevstrlen = 0;
+  sendXIMPreeditDraw(_crnt_im, _crnt_ic, caret, 0, prevstrlen, strlst.join(""), feedback);
+  prevstrlen = strlst.join("").length();
 }
 
 
@@ -227,10 +150,16 @@ InputMethod::recvXIMError()
   dsbuf.setByteOrder(sysByteOrder());
   
   char unused[4];
-  u_int16_t im, ic, flag, errcode;
+  u_int16_t im, ic, flag, errcode, errlen, errtype;
   dsbuf.readRawBytes((char*)unused, 4);   // header
-  dsbuf >> im >> ic >> flag >> errcode;
-  qWarning("XIM Error received. IM:%d IC:%d flag:%d Error Code:%d  %s:%d", im, ic, flag, errcode, __FILE__, __LINE__);
+  dsbuf >> im >> ic >> flag >> errcode >> errlen >> errtype;
+  qWarning("XIM Error received. IM:%d IC:%d flag:%d ErrorCode:%d ErrorType:%d  %s:%d",
+	   im, ic, flag, errcode, errtype, __FILE__, __LINE__);
+  if (errlen > 0) {
+    char errdetail[256] = { 0 };
+    dsbuf.readRawBytes((char*)errdetail, QMIN(errlen, sizeof(errdetail) - 1));   // header
+    qDebug("Error detail: %s", errdetail);
+  }
 }
 
 
@@ -296,32 +225,19 @@ InputMethod::recvXIMSetICFocus()
   QString strsize( ftname.section('-', 7, 7) );
   if (!strsize.isEmpty() && strsize != '*') {
     int size = strsize.toInt();
-    if (size > 0)
-	ft.setPointSize( size );
+    if (size > 0) {
+      ft.setPixelSize( size );
+      //ft.setPointSize( size );
+    }
   }
 
   qDebug("set font: %s", ft.rawName().data());
   qDebug("set font pixel size:%d  point size:%d", ft.pixelSize(), ft.pointSize());  
   _kanjiconvt->setFont(ft);
-  _kanjiconvt->clear();
   
-  // check trigger flag
-  if (_xim->triggerFlag(_crnt_im, _crnt_ic) == TRUE) {   // if on-key
-    // calculate preedit point and set im-ID and ic-ID of focus window
-    bool ok;
-    QPoint pos = calcPreeditPoint(_crnt_im, _crnt_ic, ok);
-    if ( ok ) {
-      _kanjiconvt->setPreeditPoint(pos);
-      emit triggerNotify(TRUE);
-    } else {
-      qFatal("Assert  %s:%d", __FILE__, __LINE__);
-      // Disconnect IM
-      _xim->removeIM(_crnt_im);
-      _crnt_im = _crnt_ic = 0;
-    }
-
-  } else {
-    emit triggerNotify(FALSE);
+  if ( _inputon ) {
+    setXIMInputtingEnabled( _inputon );  // Re-send XIM_SET_EVENT_MASK
+    slotDecideSegments("");     // Sends dummy string for data flush
   }
 }
 
@@ -341,8 +257,11 @@ InputMethod::recvXIMUnsetICFocus()
   qDebug("UnsetICFocus IM: %d IC: %d", im, ic);
   if (_crnt_im > 0 || _crnt_ic > 0) {
     if (im == _crnt_im && ic == _crnt_ic) {
+      // Clear characters of preedit
+      _crnt_im = _crnt_ic = 0;  // Bug fix, not to send XIM_PREEDIT_DRAW
       _kanjiconvt->clear();
-      _crnt_im = _crnt_ic = 0;
+      _crnt_im = im;
+      _crnt_ic = ic;
 
       // Garbage collection IM
       garbageCollectIM();
@@ -421,17 +340,17 @@ InputMethod::recvXIMGetICValues() const
   for (int i = 0; i < n/2; i++) {
     u_int16_t id;
     dsbuf >> id;
-    
     QByteArray value = _xim->getICValue(im, ic, id);
     dsxic << id << (u_int16_t)value.size();
     dsxic.writeRawBytes(value.data(), value.size());
-    for (int j = 0; j < pad(value.size()); j++)  dsxic << (u_int8_t)0; // unused
+    for (int j = 0; j < pad(value.size()); ++j)
+      dsxic << (u_int8_t)0; // unused
   }
   
   QByteArray data;         // reply data
   QDataStream ds(data, IO_WriteOnly);
   ds.setByteOrder(sysByteOrder());
-  
+
   ds << (u_int8_t)XIM_GET_IC_VALUES_REPLY << (u_int8_t)0 << numElements(8 + listxicattr.size())
      << im << ic << (u_int16_t)listxicattr.size()
      << (u_int16_t)0;       // unused
@@ -463,7 +382,7 @@ InputMethod::recvXIMCreateIC() const
   u_int16_t ic = _xim->createXIC(im);   // create IC
   qDebug("IM:%d  Create IC id:%d data size:%d", im, ic, buf.size());
 
-  QByteArray listofxic(n), data;
+  QByteArray listofxic(n);
   QDataStream dslistxic(listofxic, IO_ReadOnly);
   dslistxic.setByteOrder(sysByteOrder());
   dsbuf.readRawBytes(listofxic.data(), n);
@@ -471,20 +390,20 @@ InputMethod::recvXIMCreateIC() const
   while ( !dslistxic.atEnd() ) {
     u_int16_t id, len;
     dslistxic >> id >> len;
-    data.resize(len);
-    dslistxic.readRawBytes(data.data(), len);
+    QByteArray ba(len);
+    dslistxic.readRawBytes(ba.data(), len);
     dslistxic.readRawBytes((char*)unused, pad(len));  // unused
-    _xim->setICValue(im, ic, id, data);   // set IC value
+    _xim->setICValue(im, ic, id, ba);   // set IC value
   }
 
-  XClientMessageEvent cm;
-  memset(cm.data.b, 0, 20);
-  cm.data.b[0] = (u_int8_t)XIM_CREATE_IC_REPLY;
-  cm.data.s[1] = 1;   // length
-  cm.data.s[2] = im;  // set input-method-ID
-  cm.data.s[3] = ic;  // set input-context-ID
+  QByteArray data;         // reply data
+  QDataStream ds(data, IO_WriteOnly);
+  ds.setByteOrder(sysByteOrder());
 
-  sendClientMessageProtocol(_xim->commWin(im), cm);
+  ds << (u_int8_t)XIM_CREATE_IC_REPLY << (u_int8_t)0 << numElements(4)
+     << im << ic;
+
+  sendClientMessage(im, data);
   qDebug("XSendEvent XIM_CREATE_IC_REPLY");
 }
 
@@ -569,8 +488,7 @@ InputMethod::recvXIMTriggerNotify()
   u_int16_t im, ic;
   dsbuf.readRawBytes((char*)unused, 4);   // header
   dsbuf >> im >> ic >> flag >> index >> mask;
-  qDebug("XIMTriggerNotify Request IM: %d IC:%d", im, ic);
-  qDebug("flag:%lu mask:%lu", flag, mask);
+  qDebug("XIMTriggerNotify Request IM: %d IC:%d flag:%lu mask:%lu", im, ic, flag, mask);
 
   if (_crnt_im > 0 && im != _crnt_im || _crnt_ic > 0 && ic != _crnt_ic) {
     qWarning("Bad im or ic  im:%d crnt_im:%d ic:%d crnt_ic:%d", im, _crnt_im, ic, _crnt_ic);
@@ -581,42 +499,20 @@ InputMethod::recvXIMTriggerNotify()
   // Sets current IM and IC
   _crnt_im = im;
   _crnt_ic = ic;
-  _kanjiconvt->clear();   // preedit clear
 
   // Sends XIM_TRIGGER_NOTIFY_REPLY
-  XClientMessageEvent cm;
-  memset(cm.data.b, 0, 20);
-  cm.data.b[0] = (u_int8_t)XIM_TRIGGER_NOTIFY_REPLY;
-  cm.data.s[1] = 1;   // length
-  cm.data.s[2] = _crnt_im;
-  cm.data.s[3] = _crnt_ic;
-  sendClientMessageProtocol(_xim->commWin(_crnt_im), cm);
+  QByteArray data;         // reply data
+  QDataStream ds(data, IO_WriteOnly);
+  ds.setByteOrder(sysByteOrder());
+  
+  ds << (u_int8_t)XIM_TRIGGER_NOTIFY_REPLY << (u_int8_t)0 << numElements(4)
+     << _crnt_im << _crnt_ic;
+  
+  sendClientMessage(_crnt_im, data);
   qDebug("XSendEvent XIM_TRIGGER_NOTIFY_REPLY");
 
-  if ( flag ) {
-    // if off-key
-    _xim->setTriggerFlag(_crnt_im, _crnt_ic, FALSE);
-    sendXIMSetEventMask(_crnt_im, _crnt_ic);  // XIM_SET_EVENT_MASK
-    emit triggerNotify(FALSE);  // trigger signal
-    sendXIMPreeditDone(_crnt_im, _crnt_ic);
-
-  } else {
-    // if on-key
-    bool ok;
-    QPoint pos = calcPreeditPoint(_crnt_im, _crnt_ic, ok);
-    if ( ok ) {
-      _xim->setTriggerFlag(_crnt_im, _crnt_ic, TRUE);
-      sendXIMSetEventMask(_crnt_im, _crnt_ic, KeyPressMask); // XIM_SET_EVENT_MASK
-      _kanjiconvt->setPreeditPoint(pos);
-      emit triggerNotify(TRUE);   // trigger signal
-      sendXIMPreeditStart(_crnt_im, _crnt_ic);
-    } else {
-      qFatal("Assert  %s:%d", __FILE__, __LINE__);
-      // Remove IM
-      _xim->removeIM(im);
-      _crnt_im = _crnt_ic = 0;
-    }
-  }
+  emit triggerNotify( !_inputon );  // trigger signal  
+  _kanjiconvt->clear();   // preedit clear, after sendClientMessage
 }
 
 
@@ -649,15 +545,14 @@ InputMethod::recvXIMEncodingNegotiation() const
   }
   Q_ASSERT(encode == QCString("COMPOUND_TEXT"));
 
-  XClientMessageEvent cm;
-  memset(cm.data.b, 0, 20);
-  cm.data.b[0] = (u_int8_t)XIM_ENCODING_NEGOTIATION_REPLY;
-  cm.data.s[1] = 2;  // length
-  cm.data.s[2] = im;  // set input-method-ID
-  cm.data.s[3] = 0;
-  cm.data.s[4] = index;
+  QByteArray data;         // reply data
+  QDataStream ds(data, IO_WriteOnly);
+  ds.setByteOrder(sysByteOrder());
+
+  ds << (u_int8_t)XIM_ENCODING_NEGOTIATION_REPLY << (u_int8_t)0 << numElements(8)
+     << im << (u_int16_t)0 << (int16_t)index << (u_int16_t)0;
   
-  sendClientMessageProtocol(_xim->commWin(im), cm);
+  sendClientMessage(im, data);
   qDebug("XSendEvent XIM_ENCODING_NEGOTIATION_REPLY");
 }
 
@@ -739,16 +634,19 @@ InputMethod::recvSelectionRequest(const XEvent& e) const
 
 
 void
-InputMethod::recvSelectionClear()
+InputMethod::recvSelectionClear(const XEvent& e)
 {
   DEBUG_TRACEFUNC();
-  if ( _xim ) {
-    delete _xim;
-    _xim = 0;
+  if (e.xselectionclear.window == winId()
+      && e.xselectionclear.selection == _server_atom) {
+    Window newowner = XGetSelectionOwner(qt_xdisplay(), _server_atom);
+    QMessageBox::critical(0, QString("Critical Message"),
+			  QString("Received Selection Clear event.\n"
+				  "Kimera stops the service.\n\n"
+				  "Window: %1  Atom: %2\n"
+				  "New Selection Owner: %3").arg(e.xselectionclear.window).arg(e.xselectionclear.selection).arg(newowner),
+			  QMessageBox::Ok | QMessageBox::Default, 0);
   }
-
-  QMessageBox::critical(0, QString("Critical Message"), QString("Received Selection Clear event.\nKimera stops service."),
-			QMessageBox::Ok | QMessageBox::Default, 0);
 }
 
 
@@ -757,6 +655,8 @@ InputMethod::recvXIMOpen(const XEvent& e)
 {
   DEBUG_TRACEFUNC();
   qDebug("length(%d): %s", e.xclient.data.b[4], e.xclient.data.b + 5);
+  if (!e.xclient.data.b[4])
+    qWarning("Bad local name!");
 
   QByteArray listofximattr;   // LISTofXIMATTR
   QDataStream dsxim(listofximattr, IO_WriteOnly);
@@ -922,14 +822,15 @@ InputMethod::recvProperty(const XEvent& e)
     // Write the receved data to buffer
     Q_ASSERT(_buffer->isOpen());
     _buffer->writeBlock((char*)value, e.xclient.data.l[0]); 
-    XFree((char*)value);
   } else {
     qWarning("Bad response  XGetWindowProperty");
     Q_ASSERT(res == Success);
     Q_ASSERT((int)nitems == e.xclient.data.l[0]);
     Q_ASSERT(bytes_after == 0);
-    Q_CHECK_PTR(value);    
   }
+
+  if ( value )
+    XFree((char*)value);
 }
 
 
@@ -948,11 +849,7 @@ InputMethod::recvXIMForwardEvent() const
   qDebug("Forward event im:%d ic:%d, flag:%d serial:%d", im, ic, flag, serial);
 
   XEvent event;
-  char* pe = new char[buf.size() - 12];
-  dsbuf.readRawBytes(pe, buf.size() - 12);
-  xEvent2XEvent(qt_xdisplay(), pe, event);
-  delete[] pe;
-  
+  xEvent2XEvent(qt_xdisplay(), (const char*)buf.data() + 12, serial, event);
   event.xkey.window = winId();     // target for this widget
   qApp->x11ProcessEvent(&event);   // To translate button event
                                    //  see keyPressEvent function
@@ -965,30 +862,27 @@ InputMethod::keyPressEvent(QKeyEvent* e)
   DEBUG_TRACEFUNC();
 
   bool ok = TRUE;
-//if (_xim->getInputStyle(_crnt_im) == ON_THE_SPOT_STYLE) {
-    QPoint pos = calcPreeditPoint(_crnt_im, _crnt_ic, ok); 
-    if ( ok ) {
-      // Set preedit point
-      _kanjiconvt->setPreeditPoint(pos);
-    } else {
-      // Remove IM
-      _xim->removeIM(_crnt_im);
-      _crnt_im = _crnt_ic = 0;
+  QPoint pos = calcPreeditPoint(_crnt_im, _crnt_ic, ok); 
+  if ( ok ) {
+    // Set preedit point
+    _kanjiconvt->setPreeditPoint(pos);
+
+    if ( !_inputon || _kanjiconvt->processKeyEvent(*e) == FALSE ) {
+      // XIM FORWARD EVENT
+      qDebug("Send XIM_FORWARD_EVENT because of no function for the key");
+      Q_ASSERT(_buffer->buffer().size() > 0);
+      sendClientMessage(_crnt_im, _buffer->buffer());
     }
-//}
-  
-  if (ok && _kanjiconvt->processKeyEvent(*e) == FALSE && _crnt_im > 0) {
-    qDebug("Send XIM_FORWARD_EVENT because of no function for the key");
-    // XIM FORWARD EVENT
-    QByteArray buf = _buffer->buffer();   // received data
-    Q_ASSERT(buf.size() > 0);
-    sendClientMessage(_crnt_im, buf);
+  } else {
+    // Remove IM
+    _xim->removeIM(_crnt_im);
+    _crnt_im = _crnt_ic = 0;
   }
 }
 
 
 void 
-InputMethod::recvXIMResetIC() const
+InputMethod::recvXIMResetIC()
 {
   DEBUG_TRACEFUNC();
   char unused[4];
@@ -999,12 +893,11 @@ InputMethod::recvXIMResetIC() const
   dsbuf.readRawBytes((char*)unused, 4);   // header
   dsbuf >> im >> ic;
   qDebug("XIMResetIC request IM:%d IC:%d", im, ic);
-  _kanjiconvt->clear();   // preedit clear
   
   QByteArray data;
   QDataStream ds(data, IO_WriteOnly);
   ds.setByteOrder(sysByteOrder());
-  ds << (u_int8_t)XIM_RESET_IC_REPLY << (u_int8_t)0 << (u_int16_t)numElements( 4 )
+  ds << (u_int8_t)XIM_RESET_IC_REPLY << (u_int8_t)0 << (u_int16_t)numElements(4)
      << im << ic << (u_int16_t)0;
   for (int i = 0; i < pad(2); i++) {
     ds << (u_int8_t)0;   // unused
@@ -1012,6 +905,11 @@ InputMethod::recvXIMResetIC() const
   
   sendClientMessage(im, data);
   qDebug("XSendEvent XIM_RESET_IC_REPLAY");
+  
+  // preedit clear
+  _crnt_im = im;
+  _crnt_ic = ic;
+  _kanjiconvt->clear();
 }
 
 
@@ -1024,6 +922,10 @@ InputMethod::slotDecideSegments(const QString& str)
     qWarning("Bad param  im:%d ic:%d", _crnt_im, _crnt_ic);
     return;
   }
+  if (!isXIMInputtingEnabled()) {
+    qDebug("XIM Inputting Disabled");
+    return;
+  }
 
   uint len = str.local8Bit().length();
   qDebug("Decide segments:%s (%d)", str.local8Bit().data(), str.length());
@@ -1175,7 +1077,7 @@ InputMethod::sendXIMError(Window client, XIMErrorCode error) const
   ds.setByteOrder(sysByteOrder());
 
   ds << (u_int8_t)XIM_ERROR << (u_int8_t)0 << numElements(12) 
-     << (u_int16_t)0 << (u_int16_t)0 << (u_int16_t)XIM_FLAG_INVALID_IM_IC << (u_int16_t)error 
+     << (u_int16_t)0 << (u_int16_t)0 << (u_int16_t)XIM_FLAG_INVALID_IM_IC << (u_int16_t)error
      << (u_int16_t)0 << (u_int16_t)0;
 
   XClientMessageEvent cm;
@@ -1190,7 +1092,7 @@ InputMethod::sendXIMPreeditStart(u_int16_t im, u_int16_t ic) const
 {
   DEBUG_TRACEFUNC("im:%d ic:%d", im, ic);
 
-  if (_xim->getInputStyle(im) != ON_THE_SPOT_STYLE) {
+  if (_xim->getInputStyle(im, ic) != ON_THE_SPOT_STYLE) {
     qDebug("Not on-the-spot style");
     return;
   }
@@ -1225,20 +1127,17 @@ InputMethod::recvXIMPreeditStartReply() const
   u_int16_t im ,ic;  
   dsbuf.readRawBytes((char*)unused, 4);   // header
   dsbuf >> im >> ic >> res;
-
-  if ( res ) {
-    qWarning("XIMPreeditStart failed  im:%d ic:%d res:%d", im, ic, res);
-  }
+  qDebug("XIMPreeditStartReply  im:%d ic:%d  res:%d", im, ic, res);
 }
 
 
 void
-InputMethod::sendXIMPreeditDraw(u_int16_t im, u_int16_t ic, const QString& str, const QByteArray& feedback) const
+InputMethod::sendXIMPreeditDraw(u_int16_t im, u_int16_t ic, int caret, int index, int length, const QString& str, const QByteArray& feedback) const
 {
   DEBUG_TRACEFUNC("str: %s", str.local8Bit().data());
   
-  if (_xim->getInputStyle(im) != ON_THE_SPOT_STYLE) {
-    qDebug("Not on-the-spot style");
+  if (_xim->getInputStyle(im, ic) != ON_THE_SPOT_STYLE) {
+    qWarning("Not on-the-spot style");
     return;
   }
   if ( !_xim->ximPreeditStarted(im, ic) ) {
@@ -1252,14 +1151,7 @@ InputMethod::sendXIMPreeditDraw(u_int16_t im, u_int16_t ic, const QString& str,
 
   XTextProperty tp;
   memset(&tp, 0, sizeof(tp));
-  int chg_first = 0;
-  int chg_length = 0;
-  if ( str.isEmpty() ) {
-    chg_length = _preedit_str.length();
-
-  } else {
-    chg_first = _preedit_str.length();
-
+  if ( !str.isEmpty() ) {
     uint buflen = str.local8Bit().length() + 1;
     char* pstr = (char*)alloca(buflen);
     strncpy(pstr, str.local8Bit().data(), buflen);
@@ -1267,27 +1159,25 @@ InputMethod::sendXIMPreeditDraw(u_int16_t im, u_int16_t ic, const QString& str,
     Q_ASSERT(res == Success);
   }
 
-  int stat = str.isEmpty() ? 1 : 0;
-  stat |= feedback.isEmpty() ? 2 : 0;
-
   QByteArray  data;
   QDataStream ds(data, IO_WriteOnly);
   ds.setByteOrder(sysByteOrder());
-  ds << im << ic 
-     << (int)tp.nitems << chg_first << chg_length
-     << stat << (u_int16_t)tp.nitems;
+  ds << im << ic << caret << index << length;
   
-  if ( tp.value ) {
+  if (!str.isEmpty() && tp.value && tp.nitems > 0) {
+    ds << (int)0 << (u_int16_t)tp.nitems;
     ds.writeRawBytes((char*)tp.value, tp.nitems);
     XFree(tp.value);
+
+    for (int i = 0; i < pad(2 + tp.nitems); ++i)
+      ds << (u_int8_t)0;       // unused
+    
+    ds << (u_int16_t)feedback.size() << (u_int16_t)0;
+    ds.writeRawBytes(feedback.data(), feedback.size());
+  } else {
+    ds << (int)0x3;
   }
 
-  for (int i = 0; i < pad(2 + tp.nitems); ++i)
-    ds << (u_int8_t)0;       // unused
-
-  ds << (u_int16_t)feedback.size() << (u_int16_t)0;
-  ds.writeRawBytes(feedback.data(), feedback.size());
-
   QByteArray  msg;
   QDataStream dsmsg(msg, IO_WriteOnly);
   dsmsg.setByteOrder(sysByteOrder());
@@ -1304,7 +1194,7 @@ InputMethod::sendXIMPreeditDone(u_int16_t im, u_int16_t ic) const
 {
   DEBUG_TRACEFUNC();
 
-  if (_xim->getInputStyle(im) != ON_THE_SPOT_STYLE) {
+  if (_xim->getInputStyle(im, ic) != ON_THE_SPOT_STYLE) {
     qDebug("Not on-the-spot style");
     return;
   }
@@ -1465,14 +1355,12 @@ InputMethod::customEvent(QCustomEvent* e)
   if (e->type() == (QEvent::Type)KimeraApp::Selection) {
     recvSelectionEvent(e);
     return;
-
-  } else if (e->type() != (QEvent::Type)KimeraApp::XIMClientMessage) {    
+  }
+  if (e->type() != (QEvent::Type)KimeraApp::XIMClientMessage || !_xim) {
     return;
   }
 
-  if ( !_xim ) return;
-
-  XEvent *event = (XEvent*)e->data();
+  XEvent* event = (XEvent*)e->data();
   
   // Dispatch Client Message
   if (event->xclient.message_type == _xim_xconnect) {
@@ -1494,21 +1382,20 @@ InputMethod::customEvent(QCustomEvent* e)
   
   if (event->xclient.message_type == _xim_protocol && _buffer->size() > 0) {
     Q_ASSERT( !_buffer->buffer().at(1) );  // minor-opcode must be zero
-    qDebug("- Receive Message Protocol  number:%d  size:%d -",
+    qDebug("=== Received Message Protocol  number:%d  size:%d",
 	   _buffer->buffer().at(0), _buffer->buffer().size());
   
-    XEvent e = *event;    
     switch (_buffer->buffer().at(0)) {  // major-opcode of received packet
     case XIM_CONNECT:
-      recvXIMConnect( e );
+      recvXIMConnect( *event );
       break;
 
     case XIM_DISCONNECT:
-      recvXIMDisconnect( e );
+      recvXIMDisconnect( *event );
       break;
       
     case XIM_OPEN:
-      recvXIMOpen( e );
+      recvXIMOpen( *event );
       break;
       
     case XIM_CLOSE:
@@ -1613,7 +1500,7 @@ InputMethod::recvSelectionEvent(const QEvent* e)
 
   case SelectionClear:
     qDebug("Receive SelectionClear");
-    recvSelectionClear();
+    recvSelectionClear(event);
     break;
 
   case SelectionNotify:
@@ -1664,15 +1551,17 @@ InputMethod::calcPreeditPoint(u_int16_t im, u_int16_t ic, bool& ok) const
   }
   
   QPoint pos = QPoint(x, y);
-  if (_xim->getInputStyle(im) == ON_THE_SPOT_STYLE) {
-    // Adjusts point in case of on-the-spot style
+  QPoint spotpret = _xim->spotPreedit(im, ic);
+  XIMStyle style = _xim->getInputStyle(im, ic);
+  if ((style == ON_THE_SPOT_STYLE || style == ROOT_WINDOW_STYLE) && spotpret == QPoint(0, 0)) {
+    // Adjusts point in case of on-the-spot style or root-window style
     XWindowAttributes attr;
     Status s = XGetWindowAttributes(qt_xdisplay(), _xim->focusWindow(im, ic), &attr);
     if ( s ) {
       pos += QPoint(0, attr.height);
     }
   } else {
-    pos += _xim->spotPreedit(im, ic);
+    pos += spotpret;
   }
   
   qDebug("latest focus windows:%ld  IM:%u IC:%u x:%d y:%d",
@@ -1704,3 +1593,99 @@ InputMethod::checkCommWindow(u_int16_t im) const
   return res;
 }
 
+
+void
+InputMethod::registerXIMServerKimera()
+{
+  DEBUG_TRACEFUNC();
+
+  // Create Atom
+  Atom xim_servers = XInternAtom(qt_xdisplay(), "XIM_SERVERS", False);
+  qDebug("XIM_SERVERS: %lu", xim_servers);
+  Atom server_atom = XInternAtom(qt_xdisplay(), SERVER_ATOM_STRING, False);
+  qDebug("@server=kimera: %lu", server_atom);
+
+  Atom type;
+  int format;
+  ulong nitems;
+  ulong bytes_after;
+  u_int8_t* value = 0;
+
+  // Grab server
+  XGrabServer(qt_xdisplay());
+
+  // Get the Atom contained in XIM_SERVERS Property
+  int res = XGetWindowProperty(qt_xdisplay(), qt_xrootwin(), xim_servers,
+                               0, 1024, False, XA_ATOM, &type,
+                               &format, &nitems, &bytes_after, &value);
+  Q_ASSERT(res == Success);
+  Q_ASSERT(bytes_after == 0);
+
+  // Compare the gotten Atom and XIM_SERVER Atom
+  Atom tmpatom = server_atom;
+  if (res != Success || type != XA_ATOM || format != 32 || !value ) {
+    // Register XIM_SERVER Atom
+    XChangeProperty(qt_xdisplay(), qt_xrootwin(), xim_servers,
+                    XA_ATOM, 32, PropModeReplace,
+                    (u_int8_t *)&tmpatom, 1);
+    qDebug("No XIM_SERVER. Regiter IM server");
+
+  } else {
+    ulong* atoms = (ulong*)value;
+    int i;
+    for (i = 0; i < (int)nitems; i++) {
+      if (atoms[i] == server_atom) {
+        // Already registered
+        qDebug("IM server registered already. No change.");
+        break;
+      }
+    }
+
+    if (i == (int)nitems) {
+      // Register XIM_SERVER Atom
+      XChangeProperty(qt_xdisplay(), qt_xrootwin(), xim_servers,
+                      XA_ATOM, 32, PropModeAppend,
+                      (u_int8_t *)&tmpatom, 1);
+      qDebug("No Kimera. Register IM server");
+    }
+  }
+
+  if ( value )
+    XFree((char*)value);
+
+  // Ungrab server
+  XUngrabServer(qt_xdisplay());
+
+  // Flush
+  XFlush(qt_xdisplay());
+}
+
+
+void
+InputMethod::setXIMInputtingEnabled(bool enable)
+{
+  DEBUG_TRACEFUNC("enable: %d", enable);
+ 
+  _inputon = enable;  
+  if (_crnt_im > 0 && _crnt_ic > 0) {
+    if ( _inputon ) {
+      // if on-key
+      bool ok;
+      QPoint pos = calcPreeditPoint(_crnt_im, _crnt_ic, ok);
+      if ( ok ) {
+	sendXIMSetEventMask(_crnt_im, _crnt_ic, KeyPressMask); // XIM_SET_EVENT_MASK
+	_kanjiconvt->setPreeditPoint(pos);
+	sendXIMPreeditStart(_crnt_im, _crnt_ic);
+      } else {
+	qFatal("Assert  %s:%d", __FILE__, __LINE__);
+	// Remove IM
+	_xim->removeIM(_crnt_im);
+	_crnt_im = _crnt_ic = 0;
+      }
+    } else {
+      // if off-key
+      sendXIMSetEventMask(_crnt_im, _crnt_ic);  // XIM_SET_EVENT_MASK
+      sendXIMPreeditDone(_crnt_im, _crnt_ic);
+    }
+  }
+}
