@@ -3,14 +3,13 @@
 #include <X11/Xproto.h>
 #include <X11/Xmd.h>
 
-void xEvent2XEvent(Display* dis, const void* src, XEvent& e)
+void xEvent2XEvent(Display* dis, const void* src, unsigned int serial, XEvent& e)
 {
   xEvent event;
-  memset(&event, 0, sz_xEvent);
   memcpy(&event, src, sz_xEvent);
 
   e.type = event.u.u.type & 0x7f;
-  //e.xany.serial = (serial << 16) | event.u.u.sequenceNumber;
+  e.xany.serial = (serial << 16) | event.u.u.sequenceNumber;
   e.xany.display = dis;
   e.xany.send_event = event.u.u.type > 127;
   e.xkey.window = event.u.keyButtonPointer.event;
