@@ -24,7 +24,7 @@ KanjiEngine::KanjiEngine()
 {
   if (!enginelist) {
     enginelist = new QPtrList<KanjiEngine>;
-    enginelist->setAutoDelete(TRUE);
+    enginelist->setAutoDelete(FALSE);
   }
   enginelist->append(this);
 }
@@ -43,9 +43,17 @@ KanjiEngine::cleanup()
 }
 
 
+bool
+KanjiEngine::isKanjiConvEnabled() const
+{
+  return TRUE;
+}
+
+
 bool
 KanjiEngine::isConverting()
 {
+  Q_ASSERT(0);
   return FALSE;
 }
 
@@ -81,3 +89,22 @@ KanjiEngine::kanjiEngineList()
   }
   return lst;
 }
+
+
+bool
+KanjiEngine::isPredictionEnabled() const
+{
+  return FALSE;
+}
+
+
+void
+KanjiEngine::predict(const QString&, const QString&)
+{
+}
+
+
+void
+KanjiEngine::learn(const QStringList&, const QStringList&)
+{
+}
