@@ -8,6 +8,8 @@ static InputMethod* inpmethod = 0;
 KimeraApp::KimeraApp(int &argc, char **argv) : QApplication(argc, argv)
 {
   qAddPostRoutine( cleanup_ptr );   // delete object
+
+  InputMethod::registerXIMServerKimera();  // Register Kimera to X
 }
 
 //
@@ -52,3 +54,10 @@ KimeraApp::x11EventFilter(XEvent* event)
   
   return FALSE;
 }
+
+
+bool
+KimeraApp::isXIMInputtingEnabled()
+{
+  return  inputmethod()->isXIMInputtingEnabled();
+}
