@@ -1,7 +1,7 @@
 #include "preeditarea.h"
 #include "kimeraapp.h"
 #include "inputmethod.h"
-#include "ximattribute.h"
+#include "xicattribute.h"
 #include "config.h"
 #include <qlabel.h>
 #include <qrect.h>
@@ -17,7 +17,7 @@ PreeditArea::PreeditArea()
   _attentseg  = new QLabel(this, "attention_segment");
   _backseg    = new QLabel(this, "back_segment");
   _forwardseg = new QLabel(this, "forward_segment");
-
+  _caret = new QLabel(this, "caret");
   resize(0, 0);
 }
 
@@ -26,28 +26,32 @@ void
 PreeditArea::polish()
 {
   init();
-
-  _attentseg->setMargin( 1 );
+  _attentseg->setMargin( 0 );
   _attentseg->setIndent( 0 );
   _attentseg->setAutoMask( FALSE );
   _attentseg->setFrameStyle( QFrame::Plain | QFrame::Box );
   _attentseg->setLineWidth( 0 );
-  _attentseg->setAlignment( AlignAuto | AlignTop );
+  _attentseg->setMidLineWidth( 0 );
+  _attentseg->setAlignment( AlignAuto | AlignVCenter );
 
-  _backseg->setMargin( 1 );
+  _backseg->setMargin( 0 );
   _backseg->setIndent( 0 );
   _backseg->setAutoMask( FALSE );
   _backseg->setFrameStyle( QFrame::Plain | QFrame::Box );
   _backseg->setLineWidth( 0 );
-  _backseg->setAlignment( AlignAuto | AlignTop );
+  _backseg->setMidLineWidth( 0 );
+  _backseg->setAlignment( AlignAuto | AlignVCenter );
   _backseg->move(0, 0);
 
-  _forwardseg->setMargin( 1 );
+  _forwardseg->setMargin( 0 );
   _forwardseg->setIndent( 0 );
   _forwardseg->setAutoMask( FALSE );
   _forwardseg->setFrameStyle( QFrame::Plain | QFrame::Box );
   _forwardseg->setLineWidth( 0 );
-  _forwardseg->setAlignment( AlignAuto | AlignTop );  
+  _forwardseg->setMidLineWidth( 0 );
+  _forwardseg->setAlignment( AlignAuto | AlignVCenter );  
+
+  _caret->setBackgroundColor( QColor(0, 0, 0) );
 }
 
 
@@ -73,94 +77,84 @@ PreeditArea::setFont(const QFont & f)
   _attentseg->setFont( f );
   _backseg->setFont( f );
   _forwardseg->setFont( f );
+  _caret->resize(2, QFontMetrics( f ).height());
 }
 
 
 void
-PreeditArea::showText(const QStringList& strlist, uint attention)
+PreeditArea::showText(const QStringList& strlist, uint attention, uint caret_pos)
 {
-  if (attention >= strlist.count())
+  if (attention >= strlist.count()) {
+    qFatal("Bad parameter");
     return;
-  
-  // Ѵʸ
-  if ( strlist.isEmpty() ) {
-    _attentseg->setText(QString::null);
-    _backseg->setText(QString::null);
-    _forwardseg->setText(QString::null);
-
-  } else {
-    // Sets back_segment
-    QStringList sl(strlist);
-    QStringList  backsegment;
-    for (uint i = 0; i < attention; i++) {
-      backsegment << sl.first();
-      sl.pop_front();
-    }
-    backsegment.isEmpty() ? _backseg->setText(QString::null) : _backseg->setText(backsegment.join(""));
-
-    // Set text of preedit area
-    _attentseg->setText(sl.first());
-    sl.pop_front();
-    
-    // Sets forward_segment
-    sl.isEmpty() ? _forwardseg->setText(QString::null) : _forwardseg->setText(sl.join(""));
   }
+  if (strlist.isEmpty()) {
+    hide();
+    return;
+  }
+  
+  // Sets segments
+  QString str;
+  for (int i = 0; i < (int)attention; ++i)
+    str += strlist[i];
+  _backseg->setText(str);
+  _attentseg->setText(strlist[attention]);
+  str = QString::null;
+  for (int i = attention + 1; i < (int)strlist.count(); ++i)
+    str += strlist[i];
+  _forwardseg->setText(str);
 
   // Show
   switch ( KimeraApp::inputmethod()->currentXIMStyle() ) {
   case OVER_THE_SPOT_STYLE:
-    if (_attentseg->text().isEmpty()) {
-      hide();
-      break;
-    }
-
+  case ROOT_WINDOW_STYLE:
     _backseg->text().isEmpty() ? _backseg->hide() : _backseg->show();
     _attentseg->show();
     _forwardseg->text().isEmpty() ? _forwardseg->hide() : _forwardseg->show();
+    if (_state != Input) {
+      _caret->hide();
+    } else {
+      QFontMetrics fm( _attentseg->font() );
+      int pos = fm.width(_attentseg->text(), caret_pos);
+      if (caret_pos == _attentseg->text().length())
+	pos += _attentseg->margin() + 1;
+      _caret->move(QPoint(pos, 1));
+      _caret->raise();
+      _caret->show();
+    }
     show();
     adjustSize();
     break;
 
   case ON_THE_SPOT_STYLE:
-    KimeraApp::inputmethod()->clearPreedit();
-    if (_attentseg->text().isEmpty()) {
-      break;
-    }
-
     if (_state == Input) {
-      KimeraApp::inputmethod()->appendPreeditString(_attentseg->text(), InputMethod::FT_UNDERLINE);
+      KimeraApp::inputmethod()->sendPreeditString(caret_pos, _attentseg->text());
     } else {
-      if ( !_backseg->text().isEmpty())
-	KimeraApp::inputmethod()->appendPreeditString(_backseg->text(), InputMethod::FT_UNDERLINE);
-      
-      KimeraApp::inputmethod()->appendPreeditString(_attentseg->text(), InputMethod::FT_REVERSE);
-      if ( !_forwardseg->text().isEmpty() )
-	KimeraApp::inputmethod()->appendPreeditString(_forwardseg->text(), InputMethod::FT_UNDERLINE);
+      KimeraApp::inputmethod()->sendPreeditString(strlist.join("").length(), strlist, attention);
     }
     break;
-
+    
   default:
-    qWarning("Bad XIMStyle");
+    qWarning("Bad XIMStyle: %lu", KimeraApp::inputmethod()->currentXIMStyle());
     Q_ASSERT(0);
     return;
     break;
   }
-
+  
   qDebug("showText  back: %s  attention: %s  forward: %s", _backseg->text().local8Bit().data(),
 	 _attentseg->text().local8Bit().data(), _forwardseg->text().local8Bit().data());
 }
 
 
 void 
-PreeditArea::showInputingString(const QString& str)
+PreeditArea::showInputingString(const QString& str, uint caret_pos)
 {
   Q_ASSERT( !str.isEmpty() );
   _state = Input;
   _attentseg->setFont( _font[Input] );
   _attentseg->setPaletteForegroundColor( _colorlist[FgInput] );
   _attentseg->setPaletteBackgroundColor( _colorlist[BgInput] );
-
-  showText(QStringList(str));
+  showText(QStringList(str), 0, caret_pos);
 }
 
 
@@ -209,40 +203,42 @@ PreeditArea::showChangingSegmentLength(const QStringList& strlist, uint attentio
 void
 PreeditArea::adjustSize()
 {
-  if ( !_attentseg->text().isEmpty() ) {
-    _backseg->resize( _backseg->sizeHint() );
+  QWidget::adjustSize();
 
-    QPoint p(0, 0);  
+  const QSize hmargin(0, 2);
+  if ( !_attentseg->text().isEmpty() ) {
+    _backseg->resize( _backseg->sizeHint() + hmargin );
+
+    QPoint p(0, 0);
     if ( !_backseg->text().isEmpty() )
       p += QPoint(_backseg->size().width(), 0);
   
     _attentseg->move( p );
-    _attentseg->resize( _attentseg->sizeHint() );
+    _attentseg->resize( _attentseg->sizeHint() + hmargin );
     emit listPointChanged( mapToGlobal(p + QPoint(0, _attentseg->height()) + QPoint(-4, 1)) );   // Current segment pos
+    
     p += QPoint(_attentseg->size().width(), 0);
     _forwardseg->move( p );
-    _forwardseg->resize( _forwardseg->sizeHint() );
+    _forwardseg->resize( _forwardseg->sizeHint() + hmargin );
 
     if ( !_forwardseg->text().isEmpty() )
       p += QPoint(_forwardseg->size().width(), 0);
   
-    resize( QSize(p.x(), _attentseg->sizeHint().height()) );
+    resize( QSize(p.x(), _attentseg->size().height()) );
     qDebug("back: %s  attention: %s  forward: %s", _backseg->text().local8Bit().data(),
 	   _attentseg->text().local8Bit().data(), _forwardseg->text().local8Bit().data());
   }
-
-  QWidget::adjustSize();
 }
 
 
 void
 PreeditArea::movePreeditPos(const QPoint& pos)
 {
-  int y_offset = (font().pointSize() > 0) ? font().pointSize() + 3 : font().pixelSize();
-  move(pos - QPoint(0, y_offset));
+  int offset = QFontMetrics(_attentseg->font()).height() - 1;
+  move(pos - QPoint(0, offset));
 
   if (KimeraApp::inputmethod()->currentXIMStyle() == ON_THE_SPOT_STYLE)
-    emit listPointChanged(pos);   // Current segment pos
+    emit listPointChanged(pos + QPoint(12, 1));   // Current segment pos
 }
 
 
@@ -277,10 +273,11 @@ PreeditArea::forwardText() const
 void
 PreeditArea::hide()
 {
-  KimeraApp::inputmethod()->clearPreedit();
+  if (KimeraApp::inputmethod()->currentXIMStyle() == ON_THE_SPOT_STYLE) {
+    KimeraApp::inputmethod()->sendPreeditString();  // Clear string
+  }
   _attentseg->setText(QString::null);
   _backseg->setText(QString::null);
   _forwardseg->setText(QString::null);
-  
   QWidget::hide();
 }
