@@ -6,6 +6,10 @@
 #include <qpoint.h>
 #include <X11/Xlib.h>
 
+const XIMStyle  OVER_THE_SPOT_STYLE = XIMPreeditPosition  | XIMStatusNothing;
+const XIMStyle  ON_THE_SPOT_STYLE   = XIMPreeditCallbacks | XIMStatusNothing;
+const XIMStyle  ROOT_WINDOW_STYLE   = XIMPreeditNothing   | XIMStatusNothing;
+
 class XICAttribute {
 public:
   XICAttribute();
@@ -13,10 +17,9 @@ public:
 
   void        setValue(u_int16_t, const QByteArray&); 
   QByteArray  getValue(u_int16_t) const;
-  void        setTriggerFlag(bool b) { _trgr_flag = b; }
-  bool        triggerFlag() const { return _trgr_flag; }
   void        setXIMPreeditStarted(bool b) { _xim_predtstarted = b; }
   bool        ximPreeditStarted() const { return _xim_predtstarted; }
+  XIMStyle    inputStyle() const { return _inputstyle; }
   Window      focusWindow() const { return _focuswin ? _focuswin : _clientwin; }
   QCString    fontPreedit() const { return _fontset_predt; }
   void        setSpotPreedit(const QPoint& p) { _spot_predt.x = p.x(); _spot_predt.y = p.y(); }
@@ -28,9 +31,9 @@ protected:
   int   pad(int n) const { return (4 - (n % 4)) % 4; }
 
 private:
-  bool        _trgr_flag;          // trigger flag   TRUE:on-key  FALSE:off-key 
-  bool        _xim_predtstarted;   // on-the-spot preedit start flag 
- 
+  bool        _xim_predtstarted;   // on-the-spot preedit start flag
+
+  XIMStyle    _inputstyle;    // attribute ID: 0
   Window      _clientwin;     // attribute ID: 1
   ulong       _filterevents;  // attribute ID: 4
   Window      _focuswin;      // attribute ID: 5
