@@ -86,6 +86,14 @@ XIContext::getValue(u_int16_t ic, u_int16_t id) const
 }
 
 
+XIMStyle
+XIContext::inputStyle(u_int16_t ic) const
+{
+  XICAttribute* pxic = _iclist[ic];
+  return pxic ? pxic->inputStyle() : 0;
+}
+
+
 Window
 XIContext::focusWindow(u_int16_t ic) const
 {
@@ -118,22 +126,6 @@ XIContext::spotPreedit(u_int16_t ic) const
 }
 
 
-void
-XIContext::setTriggerFlag(u_int16_t ic, bool b)
-{
-  XICAttribute* pxic = _iclist[ic];
-  if ( pxic ) pxic->setTriggerFlag(b);
-}
-
-
-bool
-XIContext::triggerFlag(u_int16_t ic) const
-{
-  XICAttribute* pxic = _iclist[ic];
-  return pxic ? pxic->triggerFlag() : FALSE;
-}
-
-
 bool
 XIContext::ximPreeditStarted(u_int16_t ic) const
 {
