@@ -22,12 +22,11 @@ public:
   void         removeIC(u_int16_t ic);
   void         setValue(u_int16_t ic, u_int16_t id, const QByteArray&); 
   QByteArray   getValue(u_int16_t ic, u_int16_t id) const;
+  XIMStyle     inputStyle(u_int16_t ic) const;
   Window       focusWindow(u_int16_t ic) const;
   QString      fontPreedit(u_int16_t ic) const;
   void         setSpotPreedit(u_int16_t ic, const QPoint& p);
   QPoint       spotPreedit(u_int16_t ic) const;
-  void         setTriggerFlag(u_int16_t ic, bool b);
-  bool         triggerFlag(u_int16_t ic) const;
   bool         ximPreeditStarted(u_int16_t ic) const;
   void         setXIMPreeditStarted(u_int16_t ic, bool b);
   QValueList<u_int16_t>  icList() const;
