@@ -2,6 +2,7 @@
 #include "xicattribute.h"
 #include "xicontext.h"
 #include "kimeraglobal.h"
+#include "debug.h"
 
 QValueList<SupportAttr> XIMethod::_attr_list = QValueList<SupportAttr>();
 
@@ -107,11 +108,6 @@ XIMethod::setICValue(u_int16_t im, u_int16_t ic, u_int16_t id, const QByteArray&
   qDebug("%s: IM:%d IC:%d attributes-id:%d data-size:%d",
 	 __func__, im, ic, id, data.size());
 
-  if (id == 0) {
-    setInputStyle(im, data);
-    return;
-  } 
-  
   XIMAttribute* pxim = _imlist[im];
   if ( pxim )
     pxim->_xic.setValue(ic, id, data);
@@ -169,28 +165,12 @@ XIMethod::focusWindow(u_int16_t im, u_int16_t ic) const
 }
 
 
-void
-XIMethod::setInputStyle(u_int16_t im, const QByteArray& data) 
-{
-  XIMStyle s;
-  QDataStream ds(data, IO_ReadOnly);
-  ds.setByteOrder(sysByteOrder());
-  Q_ASSERT(data.size() == sizeof(XIMStyle));
-
-  ds >> s;
-  qDebug("client request input styles:%lx", s);
-  Q_ASSERT(s == OVER_THE_SPOT_STYLE || s == ON_THE_SPOT_STYLE);
-  XIMAttribute*  pxim = _imlist[im];
-  if ( pxim )
-    pxim->setStyle(s);
-}
-
 
 XIMStyle
-XIMethod::getInputStyle(u_int16_t im) const
+XIMethod::getInputStyle(u_int16_t im, u_int16_t ic) const
 {
   XIMAttribute*  pxim = _imlist[im];
-  return pxim ? pxim->style(): 0;
+  return pxim ? pxim->_xic.inputStyle(ic) : 0;
 }
 
 
@@ -200,31 +180,13 @@ XIMethod::queryInputStyle() const
   QByteArray  ret(0);
   QDataStream ds(ret, IO_WriteOnly);
   ds.setByteOrder(sysByteOrder());
-  ds << (u_int16_t)(sizeof(XIMStyle) * 2) << (u_int16_t)0
-     << OVER_THE_SPOT_STYLE << ON_THE_SPOT_STYLE;
-  
+  ds << (u_int16_t)(sizeof(uint) * 3) << (u_int16_t)0
+     << (uint)OVER_THE_SPOT_STYLE << (uint)ON_THE_SPOT_STYLE << (uint)ROOT_WINDOW_STYLE;
+
   return ret;
 }
 
 
-void
-XIMethod::setTriggerFlag(u_int16_t im, u_int16_t ic, bool b)
-{
-  XIMAttribute*  pxim = _imlist[im];
-  if ( pxim ) {
-    pxim->_xic.setTriggerFlag(ic, b);
-  }
-}
-
-
-bool
-XIMethod::triggerFlag(u_int16_t im, u_int16_t ic) const
-{
-  XIMAttribute*  pxim = _imlist[im];
-  return pxim ? pxim->_xic.triggerFlag(ic) : FALSE;
-}
-
-
 void
 XIMethod::setXIMPreeditStarted(u_int16_t im, u_int16_t ic, bool b)
 {
